﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using MindFusion.Diagramming;
using System.Xml;
using System.IO;
using MindFusion.Diagramming.Layout;
using MindFusion.Layout;
using System.Drawing.Imaging;


namespace TagCloud
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
		}

		RectangleF defaultBounds = new RectangleF(0, 0, 10, 20);
		ShapeNode root;
		
		private void MainForm_Load(object sender, EventArgs e)
		{
			// read the tags file
			var reader = new StreamReader("words.txt");
			string line;
			while ((line = reader.ReadLine()) != null)
			{
				// each line contains "word: frequency" entries
				var parts = line.Split(new[] { ':' });
				var word = parts[0];
				var frequency = int.Parse(parts[1]);

				// create a diagram node for each word in the file
				var node = diagram.Factory.CreateShapeNode(defaultBounds);
				node.Weight = frequency;
				node.Text = word;

				// set font size corresponding to frequency
				node.Font = new Font(
					"Arial",
					8 + (float)Math.Log(node.Weight, 1.15),
					GraphicsUnit.Point);

				// resize the node to fit text
				var size = TextRenderer.MeasureText(node.Text, node.Font);
				node.Resize(
					2 + (float)MeasureUnit.Pixel.Convert(size.Width, diagram.MeasureUnit, null),
					2 + (float)MeasureUnit.Pixel.Convert(size.Height, diagram.MeasureUnit, null));

				// show only text, hide geometry
				node.Transparent = true;

				if (diagram.Nodes.Count == 1)
				{
					// save reference to the first node
					root = node;
				}
				else
				{
					// build a tree where each node has up to six children
					diagram.Factory.CreateDiagramLink(
						diagram.Nodes[diagram.Nodes.Count / 6],
						node);
				}
			}

			// use FractalLayout for initial placement. if the file entries are sorted, each circular
			// branch will contain larger parent node centered between its smaller children
			new FractalLayout().Arrange(diagram);

			// remove the links
			while (diagram.Links.Count > 0)
				diagram.Links.RemoveAt(diagram.Links.Count - 1);

			// pull all nodes towards the root to eliminate empty space that was occupied by links
			var center = root.GetCenter();
			foreach (var node in diagram.Nodes.Where(n => n != root))
			{
				var relativePos = new Vector(center, node.GetCenter());
				var newPos = center + relativePos / 10;
				node.Move(
					newPos.X - node.Bounds.Width / 2,
					newPos.Y - node.Bounds.Height / 2);
			}

			// run SpringLayout to distribute nodes more uniformly
			var sl = new SpringLayout();
			sl.Randomize = false;
			sl.SplitGraph = false;
			sl.NodeDistance = 3;
			sl.IterationCount = 40;
			sl.Arrange(diagram);

			// remove any remaining overlaps
			RemoveOverlaps(root, 0.1f);
			sl.Arrange(diagram);

			// show everything inside view
			diagram.ResizeToFitItems(5);
			diagramView.ZoomToFit();
		}

		void RemoveOverlaps(DiagramNode modifiedNode, float minDist)
		{
			var queue = new Queue<DiagramNode>();
			queue.Enqueue(modifiedNode);

			while (queue.Count > 0)
			{
				var node = queue.Dequeue();
				var nodeCenter = node.GetCenter();
				var overlaps = FindOverlaps(node, minDist);
				foreach (var overlap in overlaps)
				{
					var ovrCenter = overlap.GetCenter();
					var ovrBounds = overlap.Bounds;
					var dx = ovrCenter.X - nodeCenter.X;
					var dy = ovrCenter.Y - nodeCenter.Y;
					if (Math.Abs(dx) > Math.Abs(dy))
					{
						// offset horizontally
						if (dx < 0)
							ovrBounds.X = node.Bounds.Left - ovrBounds.Width - minDist;
						else
							ovrBounds.X = node.Bounds.Right + minDist;
					}
					else
					{
						// offset vertically
						if (dy < 0)
							ovrBounds.Y = node.Bounds.Top - ovrBounds.Height - minDist;
						else
							ovrBounds.Y = node.Bounds.Bottom + minDist;
					}

					// shifting the node might introduce new overlaps, continue processing
					overlap.Bounds = ovrBounds;
					queue.Enqueue(overlap);
				}
			}
		}

		List<DiagramNode> FindOverlaps(DiagramNode modifiedNode, float minDist)
		{
			var bounds = modifiedNode.Bounds;
			bounds.Inflate(minDist - 1, minDist - 1);

			var overlaps = new List<DiagramNode>();
			foreach (var node in diagram.Nodes)
			{
				if (modifiedNode == node)
					continue;
				if (bounds.IntersectsWith(node.Bounds))
					overlaps.Add(node);
			}
			return overlaps;
		} 


		private void btnExport_Click(object sender, EventArgs e)
		{
			var saveDlg = new SaveFileDialog();
			saveDlg.Filter = "Image files (*.png)|*.png|All files (*.*)|*.*";
			if (saveDlg.ShowDialog() == DialogResult.OK)
			{
				var image = diagram.CreateImage();
				image.Save(saveDlg.FileName, ImageFormat.Png);
			}
		}
	}
}